#
#  FILE NAME:
#      enum.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.base.exception import BaseModuleError
from p3.base.string_helpers import IsIdentifierString, IsKeywordString, SafeFmt, UniStr


_LOG = logging.getLogger( "p3.base" )


class Enum(object):
    def __init__( self, *args ):
        super(Enum, self).__setattr__( "items", {} )
        super(Enum, self).__setattr__( "directItems", {} )
        super(Enum, self).__setattr__( "referenceItems", {} )

        nextValue = 0

        for arg in args:
            if isinstance(arg, basestring):
                name, value = arg, nextValue
            else:
                try:
                    name, value = arg
                except ValueError as e:
                    _LOG.error( "Can't unpack non string element '%s' into name and value"
                                " variables.", arg )
                    raise

            name = UniStr( name )

            if not IsIdentifierString( name ):
                raise BaseModuleError(
                    BaseModuleError.codes.enumDefinitionError,
                    SafeFmt( "'%s' enum value name is not a valid identifier.", name ) )
            if IsKeywordString( name ):
                raise BaseModuleError(
                    BaseModuleError.codes.enumDefinitionError,
                    SafeFmt( "'%s' enum value name is a python keyword.", name ) )
            if name in self.items:
                raise BaseModuleError(
                    BaseModuleError.codes.enumDefinitionError,
                    SafeFmt( "'%s' enum value name is used more than once.", name ) )

            if isinstance(value, basestring):
                self.referenceItems[name] = value
            else:
                self.directItems[name] = value
                self.items[name] = value
                self.__dict__[name] = value
                nextValue = value + 1

        for referer, referee in self.referenceItems.items():
            self.items[referer] = self._ResolveReference( referer )

        super(Enum, self).__setattr__( "values", tuple(set(self.directItems.values())) )


    # ===[ Utilities ]=========================================================================== #

    def GetNames( self, value, includeReferences=False ):
        return tuple(self._LookupNames( value, includeReferences ))


    def GetNamesAsString( self, value, includeReferences=False ):
        l = self._LookupNames( value, includeReferences )
        l.sort()
        return ','.join( l )


    def GetValues( self ):
        return self.values


    # ===[ Item access ]========================================================================= #

    def __contains__( self, element ):
        if isinstance( element, basestring ):
            return element in self.items
        else:
            return element in self.values


    def __getattr__( self, name ):
        if name in self.items:
            return self.items[name]
        else:
            raise AttributeError( name )


    def __setattr__( self, name, value ):
        raise BaseModuleError(
            BaseModuleError.codes.enumConstError,
            "Enum object doesn't support direct attribute assignments." )


    def __delattr__( self, name ):
        raise BaseModuleError(
            BaseModuleError.codes.enumConstError,
            "Enum object doesn't support attribute deletion." )


    def __getitem__( self, name ):
        if not isinstance( name, basestring ):
            raise TypeError( SafeFmt( "Key '%s' is of inproper type %s", name, type(name).__name__ ) )

        name = UniStr( name )
        

        if name in self.items:
            return self.items[name]
        else:
            raise IndexError( "Item '%s' not found.", name )


    def __setitem__( self, name, value ):
        raise BaseModuleError(
            BaseModuleError.codes.enumConstError,
            "Enum item value can't be changed." )


    def __delitem__( self, name ):
        raise BaseModuleError(
            BaseModuleError.codes.enumConstError,
            "Enum item can't be deleted." )


    def __len__( self ):
        return len(self.items)


    # ===[ Internals ]=========================================================================== #

    def _LookupNames( self, value, includeReferences ):
        items = (self.items.items() if includeReferences else self.directItems.items() )

        return [n for n, v in items if v==value]


    def _ResolveReference( self, referer ):
        refTrace = []
        initRef = referer

        def Resolve( currRef ):
            refTrace.append( currRef )

            if currRef in self.directItems:
                return self.directItems[currRef]
            elif currRef in self.referenceItems:
                nextRef = self.referenceItems[currRef]

                if nextRef == currRef:
                    raise BaseModuleError(
                        BaseModuleError.codes.enumDefinitionError,
                        SafeFmt( "Item '%s' refers itself.", nextRef ) )
                elif nextRef in refTrace:
                    raise BaseModuleError(
                        BaseModuleError.codes.enumDefinitionError,
                        SafeFmt( "Cyclic reference detected: '%s' reference resolves to itself"
                                 " through %s.", initRef, ", ".join( refTrace + [nextRef] ) ) )
                else:
                    return Resolve( nextRef )
            else:
                raise BaseModuleError(
                    BaseModuleError.codes.enumDefinitionError,
                    SafeFmt( "Unresolved '%s' reference to '%s'.", refTrace[-1], currRef ) )

        return Resolve( referer )
                    
