#
#  FILE NAME:
#      import_helpers.py
#
#  REFERENCE(S):
#      * http://docs.python.org/library/functions.html#__import__
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library modules:
import inspect
import logging
import re
import sys

# P3 modules:
from p3.base.exception import BaseModuleError
from p3.base.string_helpers import SafeFmt


_LOG = logging.getLogger( "p3.base" )


def ImportModule( modulePath ):
    if not isinstance( modulePath, basestring ):
        raise TypeError( "modulePath argument is expected to be a string." )

    if not modulePath:
        raise ValueError( "modulePath is empty." )

    try:
        __import__( modulePath, globals(), locals(), [], -1 )
    except ImportError as e:
        _LOG.error( "Following exception occured when attempted to import '%s' module:\n%s",
                    modulePath, str(e) )
        raise

    return sys.modules[modulePath]
    


def ImportObject( objectPath ):
    if not isinstance( objectPath, basestring ):
        raise TypeError( "modulePath argument is expected to be a string." )

    match = re.match( "^(?P<module>(?:\w+\.)*\w+)\.(?P<object>\w+)$", objectPath )

    if match is None:
        msg = SafeFmt( "Couldn't parse object path ('%s')", objectPath )
        _LOG.error( msg )
        raise BaseModuleError( BaseModuleError.codes.invalidImportPath, msg )

    try:
        module = __import__(
            match.group('module'), globals(), locals(), [match.group('object')], -1 )
    except ImportError as e:
        _LOG.info( "Following exception occured when attempted to import '%s' object:\n%s",
                    objectPath, str(e) )
        raise

    if match.group('object') not in module.__dict__:
        msg = SafeFmt( "Couldn't import '%s' object from the '%s' module.",
                       match.group('object'), match.group('module') )
        _LOG.error( msg )
        raise ImportError( msg )

    return module.__dict__[match.group('object')]


def IsModuleAvailable( modulePath ):
    try:
        ImportModule( modulePath )
        return True
    except ImportError as e:
        return False


def ChallengeModule( modulePath, extraMsg=None ):
    try:
        ImportModule( modulePath )
    except ImportError as e:
        if extraMsg is None:
            caller = inspect.currentframe().f_back
            extraMsg = 'expected by the {0:s} module'.format( caller.f_globals['__name__'] )

        msg = "Couldn't import required '{0:s}' module ({1:s}).".format( modulePath, extraMsg )

        raise BaseModuleError(
            BaseModuleError.codes.requiredModuleNotAvailable, msg )
