#
#  FILE NAME:
#      settings.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# P3 imports:
from p3.base.string_helpers import IsAllCapsIdentifierString, IsInternalIdentifierString, SafeFmt
from p3.conf.error import ConfModuleError



class Settings( object ):
    @classmethod
    def CreateLocked( cls, settingDict ):
        return cls( settingDict )

    @classmethod
    def CreateUnlocked( cls, settingDict ):
        s = cls( settingDict )
        s._isLocked = False
        return s

    def __init__( self, settingDict={} ):
        self._isLocked = False
        self._settings = {}
        #super( Settings, self ).__setattr__( "_isLocked", False )
        #super( Settings, self ).__setattr__( "_settings", {} )

        # Check if all setting names are valid all caps identifiers:
        for name in settingDict:
            if not self.IsValidSettingName( name ):
                raise ConfModuleError(
                    ConfModuleError.codes.invalidSettingName,
                    SafeFmt( "'%s' is not a valid, all caps, non-internal identifier.", name ) )

           
        for name, value in settingDict.items():
            self._settings[name] = value

        self._isLocked = True


    @classmethod
    def IsValidSettingName( cls, name ):
        return IsAllCapsIdentifierString( name ) and not IsInternalIdentifierString( name )


    def Lock( self ):
        self._isLocked = True


    def IsLocked( self ):
        return self._isLocked


    def __contains__( self, item ):
        return item in self._settings


    def __getattr__( self, name ):
        if name in self._settings:
            return self._settings[name]
        else:
            raise ConfModuleError( ConfModuleError.codes.unknownSettingName,
                                   SafeFmt( "Unknown setting name: '%s'", name ) )


    def __setattr__( self, name, value ):
        if IsAllCapsIdentifierString( name ) and not IsInternalIdentifierString( name ):
            self.SetSetting( name, value )
        else:
            super( Settings, self ).__setattr__( name, value )
        


    def __delattr__( self, name ):
        self.DeleteSetting( name )


    def __getitem__( self, name ):
        if name in self._settings:
            return self._settings[name]
        else:
            raise ConfModuleError( ConfModuleError.codes.unknownSettingName,
                                   SafeFmt( "Unknown setting name: '%s'", name ) )


    def __setitem__( self, name, value ):
        self.SetSetting( name, value )


    def __delitem__( self, name ):
        self.DeleteSetting( name )


    def __len__( self ):
        return len(self._settings)


    def GetSetting( self, name, default=None ):
        if name in self._settings:
            return self._settings[name]
        else:
            return default


    def SetSetting( self, name, value ):
        if self._isLocked:
            raise ConfModuleError( ConfModuleError.codes.settingsLocked, "Settings are locked." )
        elif not IsAllCapsIdentifierString( name ) or IsInternalIdentifierString( name ):
            raise ConfModuleError(
                ConfModuleError.codes.invalidSettingName,
                SafeFmt( "'%s' is not a valid, all caps, non-internal identifier.", name ) )
        else:
            self._settings[name] = value


    def DeleteSetting( self, name ):
        if self._isLocked:
            raise ConfModuleError( ConfModuleError.codes.settingsLocked, "Settings are locked." )
        elif name in self._settings:
            del self._settings[name]
        else:
            raise ConfModuleError( ConfModuleError.codes.unknownSettingName,
                                   SafeFmt( "Unknown setting name: '%s'", name ) )


    def GetSettingCount( self ):
        return len(self._settings)


    def Update( self, updatingDict ):
        if self._isLocked:
            raise ConfModuleError( ConfModuleError.codes.settingsLocked, "Settings are locked." )
        else:
            for name, value in updatingDict.items():
                self.SetSetting( name, value )
