#
#  FILE NAME:
#      unique.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.data.error import DataModuleError
from p3.data.idx.base import BaseIndex


_LOG = logging.getLogger( "p3.data" )


class UniqueIndex(BaseIndex):
    def __init__( self, table, *indexCols ):
        super( UniqueIndex, self ).__init__( table )
        self._indexCols = table.ResolveColumns( *indexCols )
        self._rows = {}

    def MakeKey( self, row ):
        cells = row.cells # the property getter constructs a new tuple
        return tuple(cells[c.index].value for c in self._indexCols)


    def _FormatKeyInfo( self, row ):
        cells = row.cells # the property getter constructs a new tuple
        def vfmt( col ):
            if col.name is None:
                return "'{0:s}'".format( str(cells[col.index].value) )
            else:
                return "'{0:s}'='{1:s}'".format( col.name, str(cells[col.index].value) )

        return ', '.join( vfmt(c) for c in self._indexCols )


    def Get( self, *values ):
        assert len(values) == len(self._indexCols)

        try:
            return (self._rows[values],)
        except KeyError as e:
            return ()


    def IndexRows( self, *rows ):
        # Generate items for all indexed rows:
        newRows = {}

        for row in rows:
            assert row.table is self._table

            key = self.MakeKey( row )
            if (key in self._rows) or (key in newRows):
                raise DataModuleError(
                    DataModuleError.codes.rowNotUnique,
                    "Uniqueness check failed ({0:s}).".format( self._FormatKeyInfo(row) ) )
            else:
                newRows[key] = row

        # Uniqueness is assured now - merge newRows with _rows:
        self._rows.update( newRows )


    def ForgetAll( self ):
        self._rows.clear()


    def ForgetRows( self, *rows ):
        keys = []

        for row in rows:
            assert row.table is self._table

            key = self.MakeKey( row )
            if key in self._rows:
                keys.append( key )
            else:
                raise DataModuleError(
                    DataModuleError.codes.rowNotFound,
                    "Row ({0:s}) is not indexed.".format( self._FormatKeyInfo(row) ) )

        for key in keys:
            del self._rows[key]
