#
#  FILE NAME:
#      row.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging
import numbers

# P3 imports:
from p3.base.string_helpers import SafeFmt
from p3.data.cell import Cell
from p3.data.error import DataModuleError

_LOG = logging.getLogger( "p3.data" )


class Row(object):
    def __init__( self, table ):
        self._table = table
        self._cells = [Cell(col, self) for col in table.cols]


    @classmethod
    def IsCellLike( cls, cell ):
        return True

    # ===[ Operations ]========================================================================== #

    def AddColumn( self, col, value ):
        assert col.name is None or self._table.FindColumn( col.name ) is col

        cell = Cell( col, self, value )
        self._cells.append( cell )


    def FillDefault( self ):
        for cell in self._cells:
            cell.value = cell.col.default


    def FillByPosition( self, cellValues ):
        for cell, value in zip(self._cells, cellValues):
            cell.value = value


    def FillByName( self, namedValues ):
        for name, value in namedValues:
            idx = self._table.GetColumn( name=name ).index
            self._cells[idx].value = value

    def RemoveColumn( self, colIdx ):
        assert (colIdx >= 0) and (colIdx < len(self._cells))

        cell = self._cells.pop(colIdx)
        cell.row = None


    # ===[ Container ]=========================================================================== #

    def __contains__( self, key ):
        if isinstance( key, basestring ):
            return self.table.FindColumn( name=key ) is not None
        else:
            raise TypeError( SafeFmt( "The 'keys' argument is of type %s while only string is"
                                      " allowed.", type(key) ) )

    def __len__( self ):
        return len(self._cells)


    def __getitem__( self, key ):
        return self.GetCell( key ).value


    def __setitem__( self, key, value ):
        self.GetCell( key ).value = value


    def GetCell( self, key ):
        if isinstance( key, basestring ):
            cellIdx = self.table.GetColumn( name=key ).index

            assert cellIdx < len(self._cells)

            return self._cells[cellIdx]
        elif isinstance( key, numbers.Integral ):
            if (key >= 0) and (key < len(self._cells)):
                return self._cells[key]
            else:
                raise DataModuleError( DataModuleError.codes.invalidColumnIndex,
                                       SafeFmt( "Column index (%s) is out of valid range <0,%d)",
                                                key, len(self._cells) ) )
        else:
            raise TypeError( SafeFmt( "The 'keys' argument is of type %s while only string or"
                                      " integer is allowed.", type(key) ) )

    # ===[ Properties & Conversions ]============================================================ #

    def __str__( self ):
        return ', '.join( [repr(c.value) for c in self._cells] )


    def cells():
        def fget( self ):
            return tuple(self._cells)
        def fset( self, cells ):
            assert self._cells is None, "cells property is already initialized"
            assert len(cells) == len(self._table.cols)
            assert not [c for c in cells if not self.IsCellLike(c)]
            self._cells = [c for c in cells]
        return locals()
    cells = property(**cells())


    def table():
        def fget( self ):
            return self._table
        return locals()
    table = property( **table() )
