#
#  FILE NAME:
#      table.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import copy
import logging
import numbers

# P3 imports:
from p3.base.string_helpers import SafeFmt
from p3.data.column import Column
from p3.data.error import DataModuleError


_LOG = logging.getLogger( "p3.data" )


class Table(object):

    # ===[ Construction ]======================================================================== #

    @classmethod
    def CreateDummy( cls ):
        return cls()

    @classmethod
    def CreateNew( cls, *columns ):
        t = cls()

        for i, c in enumerate(columns):
            if isinstance( c, basestring ):
                t.AddColumn( Column( c ) )
            elif cls.IsColumnLike( c ):
                t.AddColumn( c.Clone() )
            else:
                raise TypeError( SafeFmt( "Parameter %d is of type %s which is not a basestring"
                                          " nor a Column like object.", i, type(c).__name__ ) )
        return t

    @classmethod
    def CreateFromCursor( cls, cursor ):
        from p3.data.row_constructors import NamedRowConstructor

        table = cls.CreateNew( *[d[0] for d in cursor.description] )

        row = cursor.fetchone()

        while row is not None:
            table.AddRow( NamedRowConstructor( row.items() ) )
            row = cursor.fetchone()

        return table


    @classmethod
    def CreateFromRows( cls, rows ):
        from p3.data.row_constructors import OrderedRowConstructor

        if not rows:
            return cls.CreateDummy()

        table = cls.CreateNew( *rows[0].table.cols )

        for row in rows:
            table.AddRow( OrderedRowConstructor( tuple(c.value for c in row.cells) ) )

        return table


    def __init__( self ):
        self._colIdxLut = {}
        self._cols = []
        self._rows = []


    def Clone( self ):
        return self.CreateFromRows( self.rows )

    # ===[ Column Operations ]=================================================================== #
    ## \name Column Operations
    #  \{

    ## \param valueCb Callable taking a row for which a new cell value has to be determined as
    #      its only parameter.
    #  \todo Implement some protection against adding of the same column to multiple tables (column
    #        can't be shared after introduction of the Column.index property).
    def AddColumn( self, col, valueCb=None ):
        assert self.IsColumnLike( col )

        ## \note If the valueCb parameter is specified, values of added column's cells are
        #        determined using the callback  before any table structure modification is made. It
        #        allows the table to stay in a coherent state in case of an unhandled exception
        #        raised in the callback.
        values = (tuple(valueCb(r) for r in self._rows) if valueCb is not None else None)

        col.index = len(self._cols)
        self._cols.append( col )
        if col.name is not None:
            if col.name in self._colIdxLut:
                # Consider removing the col object from the _cols list in this case to
                # leave the table in coherent state after the exception is raised.
                raise DataModuleError(
                    DataModuleError.codes.duplicateColumnName,
                    SafeFmt( "Couldn't add column named '%s' because such column already exists in"
                             " the table.", col.name ) )
            else:
                self._colIdxLut[col.name] = col.index

        if values is None:
            for row in self._rows:
                row.AddColumn( col, col.default )
        else:
            for row, value in zip(self._rows, values):
                row.AddColumn( col, value )

    ## Return a column object either by name or index.
    #  \param name Column name.
    #  \param idx Column index.
    #  \note Parameters are exclusive, i.e. they mustn't be specified together.
    #  \throws DataModuleError Raised when specified column doesn't exist in this table.
    def GetColumn( self, name=None, idx=None ):
        assert (name,idx).count(None) == 1, "Only one of parameters may be specified."

        if name is not None:
            assert isinstance( name, basestring )

            try:
                idx = self._colIdxLut[name]
            except KeyError as e:
                raise DataModuleError(
                    DataModuleError.codes.columnNotFound,
                    "Column named '{0:s}' doesn't exist in this table.".format( name ) )

            assert self.IsValidColumnIdx(idx)

            return self._cols[idx]
        else:
            assert isinstance( idx, numbers.Integral )

            try:
                return self._cols[idx]
            except IndexError as e:
                raise DataModuleError(
                    DataModuleError.codes.columnNotFound,
                    "Invalid column index ({0:d}).".format( idx ) )

        assert False, "This point mustn't ever be reached."


    ## Try to find a column object either by name or index.
    #  \param name Column name.
    #  \param idx Column index.
    #  \return Column object when the specified column was found. None otherwise.
    #  \see GetColumn
    def FindColumn( self, name=None, idx=None ):
        try:
            return self.GetColumn( name, idx )
        except DataModuleError as e:
            return None


    def IsValidColumnIdx( self, i ):
        return 0 <= i < len(self._cols)


    def RemoveColumn( self, col ):
        prevIdx = self.ResolveColumns(col)[0].index

        for row in self._rows:
            row.RemoveColumn( prevIdx )

        col = self._cols.pop(prevIdx)

        # Remove entry from the _colIdxLut table:
        if col.name in self._colIdxLut:
            del self._colIdxLut[col.name]

        # Update indices of columns following the removed one:
        for idx, col in enumerate(self._cols[prevIdx:],prevIdx):
            if col.name is not None:
                self._colIdxLut[col.name] = idx
            col.index = idx


    ## Translate list of column names, column indices and column objects into a list of column
    #  indices. Order of specification will be kept.
    def ResolveColumns( self, *spec ):
        cols = []
        for si, s in enumerate(spec):
            if isinstance( s, basestring ):
                cols.append( self.GetColumn(name=s) )
            elif isinstance( s, numbers.Integral ):
                cols.append( self.GetColumn(idx=s) )
            elif s in self._cols:
                cols.append( s )
            else:
                if self.IsColumnLike( s ):
                    raise DataModuleError(
                        DataModuleError.codes.invalidColumnSpec,
                        SafeFmt( "Column like object at position %d doesn't seem to belong to"
                                 " this table.", si ) )
                else:
                    raise DataModuleError(
                        DataModuleError.codes.invalidColumnSpec,
                        SafeFmt( "Column specification at position %d is of invalid type.", si ) )

        return cols

    ## Verify that the col object provides a necessary interface to allow its use as a column
    #  object.
    @classmethod
    def IsColumnLike( cls, col ):
         return hasattr(col, 'default') and hasattr(col, 'name')

    ## \}

    # ===[ Row Operations ]====================================================================== #

    def AddRow( self, rowConstructor ):
        assert self.IsRowConstructorLike( rowConstructor )

        r = rowConstructor.Create( self )
        self._rows.append( r )
        return r

    ## \todo Provide some indexing support.
    def FindRows( self, expression ):
        return tuple( r for r in self.rows if expression.Test(r) )

    @classmethod
    def IsRowConstructorLike( cls, rc ):
        return hasattr( rc, "Create" )

    def SortRows( self, *cols ):
        indices = tuple( c.index for c in self.ResolveColumns( *cols ) )
        return sorted( self.rows, key=lambda r: tuple(r[i] for i in indices) )


    # ===[ Properties & Conversions ]============================================================ #

    def __str__( self ):
        return '\n'.join( [str(r) for r in self._rows] )


    def cols():
        def fget( self ):
            return tuple(self._cols)
        return locals()
    cols = property(**cols())


    def rows():
        def fget( self ):
            return tuple(self._rows)
        return locals()
    rows = property(**rows())
