#
#  FILE NAME:
#      connection.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.base import SafeFmt
from p3.db.dbms import DBMS, GetDbmsIdByName
from p3.db.error import DbModuleError


_LOG = logging.getLogger( "p3.db" )


class DbConnection( object ):
    @classmethod
    def CreateFromSetting( cls, settings, settingName ):
        dbSetting = settings.GetSetting( settingName )
        if dbSetting is None:
            raise DbModuleError( DbModuleError.codes.invalidDbSetting,
                                 SafeFmt( "Couldn't find following database connection setting:"
                                          " %s", settingName ) )

        return cls.CreateFromDict( dbSetting )

    ## Create DbConnection object from standard connection specification dictionary. Following
    #  dictionary items are required: 'dbms' - database management system name (translated by
    #  the GetDbmsIdByName function to values of the DBMS enum; 'name' - database name
    #  (optional in case of MySQL); 'user' - database user name. Optional parameters include:
    #  'host' - database server host; 'port' - database server port; 'pass' - database password.
    @classmethod
    def CreateFromDict( cls, d ):
        dbmsName = d.get( 'dbms' )
        dbName   = d.get( 'name' )
        dbUser   = d.get( 'user' )

        if dbmsName is None:
            raise DbModuleError( DbModuleError.codes.invalidDbSetting,
                                 "Invalid connection specification: missing 'dbms' item (database"
                                 " management system name)." )

        dbms = GetDbmsIdByName(dbmsName)

        if (dbName is None) and (dbms <> DBMS.mySql):
            raise DbModuleError( DbModuleError.codes.invalidDbSetting,
                                 "Invalid connection specification: missing 'name' item (database"
                                 " name)." )
        if dbUser is None:
            raise DbModuleError( DbModuleError.codes.invalidDbSetting,
                                 "Invalid connection specification: missing 'user' item (database"
                                 " user name)." )

        spec = {
            'dbHost': d.get( 'host' ),
            'dbPort': d.get( 'port' ),
            'dbPass': d.get( 'pass' ) }

        return cls( dbms, dbName, dbUser, **spec )


    def __init__( self, dbms, dbName, dbUser, dbHost=None, dbPort=None, dbPass=None ):
        assert dbms in DBMS
        self.dbms   = dbms
        self.dbName = dbName
        self.dbUser = dbUser
        self.dbHost = dbHost
        self.dbPort = dbPort
        self.dbPass = dbPass
        self.dbConn = None
        self.dbCursor = None


    def Connect( self ):
        if self.dbms == DBMS.mySql:
            self.dbConn = self._MySqlConnect()
        elif self.dbms == DBMS.pgSql:
            self.dbConn = self._PgSqlConnect()
        else:
            raise DbModuleError( DbModuleError.codes.dbmsNotSupported,
                                 SafeFmt( "Unsupported database management system: '%s'",
                                          '/'.join( DBMS.GetNames( self.dbms ) ) ) )

    def Cursor( self, recreate=False ):
        if self.dbConn is None:
            self.Connect()

        if self.dbCursor is not None and not recreate:
            return self.dbCursor
        else:
            _LOG.debug( "Creating new database cursor." )

        if self.dbms == DBMS.mySql:
            import MySQLdb
            self.dbCursor = self.dbConn.cursor( MySQLdb.cursors.DictCursor )
        elif self.dbms == DBMS.pgSql:
            import psycopg2
            import psycopg2.extras
            self.dbCursor = self.dbConn.cursor( cursor_factory=psycopg2.extras.DictCursor )
        else:
            raise DbModuleError( DbModuleError.codes.dbmsNotSupported,
                                 SafeFmt( "Unsupported database management system: '%s'",
                                          '/'.join( DBMS.GetNames( self.dbms ) ) ) )
        return self.dbCursor

    def ReleaseCursor( self ):
        self.dbCursor = None


    def GetErrorClass( self ):
        if self.dbms == DBMS.mySql:
            import MySQLdb
            return MySQLdb.Error
        if self.dbms == DBMS.pgSql:
            import psycopg2
            return psycopg2.Error
        else:
            msg = SafeFmt( "Unsupported database management system: '%s'",
                           '/'.join( DBMS.GetNames( self.dbms ) ) )
            raise DbModuleError( DbModuleError.codes.dbmsNotSupported, msg )


    def GetWarningClass( self ):
        if self.dbms == DBMS.mySql:
            import MySQLdb
            return MySQLdb.Warning
        if self.dbms == DBMS.pgSql:
            import psycopg2
            return psycopg2.Warning
        else:
            msg = SafeFmt( "Unsupported database management system: '%s'",
                           '/'.join( DBMS.GetNames( self.dbms ) ) )
            raise DbModuleError( DbModuleError.codes.dbmsNotSupported, msg )


    def GetExecutedQuery( self ):
        if self.dbCursor is None:
            raise DbModuleError( DbModuleError.codes.cursorNotUsed, "Cursor wasn't created." )

        if self.dbms == DBMS.mySql:
            if not hasattr( self.dbCursor, "_last_executed" ):
                raise DbModuleError( DbModuleError.codes.cursorNotUsed,
                                     "No query was executed with the current cursor." )
            return self.dbCursor._last_executed
        elif self.dbms == DBMS.pgSql:
            if self.dbCursor.query is None:
                raise DbModuleError( DbModuleError.codes.cursorNotUsed,
                                     "No query was executed with the current cursor." )
            return self.dbCursor.query
        else:
            msg = SafeFmt( "Unsupported database management system: '%s'",
                           '/'.join( DBMS.GetNames( self.dbms ) ) )
            raise DbModuleError( DbModuleError.codes.dbmsNotSupported, msg )


    def BuildArgs( self, mandatory, optional ):
        for n, v in mandatory:
            if v is None:
                raise DbModuleError( DbModuleError.codes.mandatoryArgNotSpecified, n )

        return dict(mandatory + tuple((n,v) for n,v in optional if v is not None))

    def _MySqlConnect( self ):
        import MySQLdb

        mandatory = (
            ('user', self.dbUser),)

        optional = (
            ('db', self.dbName),
            ('host', self.dbHost),
            ('port', self.dbPort),
            ('passwd', self.dbPass))
        return MySQLdb.connect( **self.BuildArgs( mandatory, optional ) )


    def _PgSqlConnect( self ):
        import psycopg2

        mandatory = (
            ('database', self.dbName),
            ('user',     self.dbUser))

        optional = (
            ('password', self.dbPass),
            ('host',     self.dbHost),
            ('port',     self.dbPort))

        return psycopg2.connect( **self.BuildArgs( mandatory, optional ) )
