#
#  FILE NAME:
#      execute.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging
import subprocess

# P3 imports:
from p3.data import Table
from p3.db.dbms import DBMS
from p3.db.error import DbModuleError


_LOG = logging.getLogger( "p3.db" )


## Execute specified query using MySQLdb interface.
def ExecQuery( dbConn, query, args=None ):
    try:
        dbConn.Cursor().execute( query, args )
    except (dbConn.GetErrorClass(), dbConn.GetWarningClass()) as e:
        _LOG.error( "Database API exception occured during execution of a query.\n"
                    "Query:\n%s\nArguments:\n%s\nException:\n%s", query, str(args), str(e) )
        raise
    _LOG.info( "Executed query:\n%s", dbConn.GetExecutedQuery() )

    if dbConn.Cursor().description is None:
        # This attribute [description] will be None for operations that
        # do not return rows [...]
        return None
    else:
        return Table.CreateFromCursor( dbConn.Cursor() )


## Execute specified query using spawned mysql process.
def ExecQuerySh( dbConn, query ):
    if dbConn.dbms == DBMS.mySql:
        connArgs = (('--database=%s', dbConn.dbName),
                    ('--host=%s',     dbConn.dbHost),
                    ('--password=%s', dbConn.dbPass),
                    ('--port=%s',     dbConn.dbPort),
                    ('--user=%s',     dbConn.dbUser))
        args = ['mysql']
        args.extend( [o % v for o, v in connArgs if v is not None] )

        sp = subprocess.Popen(
            args, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE )
        outData, errData = sp.communicate( query )

        if sp.returncode:
            _LOG.error( "Script execution failed with error code %d and following messages:\n%s",
                        sp.returncode, outData )
            raise DbModuleError( DbModuleError.codes.shellExecutionError, errData )
    else:
        raise DbModuleError( DbModuleError.codes.dbmsNotSupported,
                             SafeFmt( "Unsupported database management system: '%s'",
                                      '/'.join( DBMS.GetNames( dbConn.dbms ) ) ) )
