#
#  FILE NAME:
#      check.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.base import SafeFmt
from p3.db.dbms import DBMS
from p3.db.error import DbModuleError
from p3.db.execute import ExecQuery


_LOG = logging.getLogger( "p3.db" )


def DbExists( mngConn, dbName ):
    assert isinstance( dbName, str )

    if mngConn.dbms == DBMS.mySql:
        result = MySqlDbExists( mngConn, dbName )
    elif mngConn.dbms == DBMS.pgSql:
        result = PgSqlDbExists( mngConn, dbName )
    else:
        msg = SafeFmt( "Unsupported database management system: '%s'",
                       '/'.join( DBMS.GetNames( mngConn.dbms ) ) )
        raise DbModuleError( DbModuleError.codes.dbmsNotSupported, msg )

    _LOG.info( "Database '%s' %s exists...", dbName, ("" if result else "doesn't") )

    return result


def MySqlDbExists( mngConn, dbName ):
    query = \
        "SELECT schema_name FROM information_schema.schemata WHERE (schema_name = %s);"
    return bool(ExecQuery( mngConn, query, (dbName,) ).rows)


def PgSqlDbExists( mngConn, dbName ):
    query = \
        "SELECT datname FROM pg_database WHERE (datname = %s);"
    return bool(ExecQuery( mngConn, query, (dbName,) ).rows)


def TableExists( dbConn, tableName ):
    assert isinstance( tableName, str )

    if dbConn.dbms == DBMS.mySql:
        result = MySqlTableExists( dbConn, tableName )
    elif dbConn.dbms == DBMS.pgSql:
        result = PgSqlTableExists( dbConn, tableName )
    else:
        msg = SafeFmt( "Unsupported database management system: '%s'",
                       '/'.join( DBMS.GetNames( dbConn.dbms ) ) )
        raise DbModuleError( DbModuleError.codes.dbmsNotSupported, msg )

    _LOG.info( "Table '%s.%s' %s exists...", dbConn.dbName, tableName,
               ("" if result else "doesn't") )

    return result


## \param schemaName table_schema column of the information_schema.tables table.
def MySqlTableExists( dbConn, tableName ):
    query = \
        """
        SELECT
            1
        FROM
            information_schema.tables
        WHERE
            (table_schema = %s) AND
            (table_name = %s);
        """

    return bool(ExecQuery( dbConn, query, (dbConn.dbName, tableName) ).rows)

## \param schemaName schemaname column of the pg_catalog.pg_tables table.
def PgSqlTableExists( dbConn, tableName ):
    query = \
        """
        SELECT
            1
        FROM
            pg_tables
        WHERE
            (schemaname = 'public') AND
            (tablename = %s);
        """

    return bool(ExecQuery( dbConn, query, (tableName, ) ).rows)
