#
#  FILE NAME:
#      create.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.base import SafeFmt
from p3.db.dbms import DBMS
from p3.db.error import DbModuleError
from p3.db.manage.check import DbExists
from p3.db.manage.drop import DropDb


_LOG = logging.getLogger( "p3.db" )


## \param mngConn DbConnection object which will be used for database management tasks;
#  \param dbName name of a database to be created;
#  \param dropExisting flag indicating that the function should drop existing database before
#      creation attempt;
def CreateDb( mngConn, dbName, dropExisting=False ):
    _LOG.info( "Creating '%s' database...", dbName )

    assert isinstance( dbName, str )

    if dropExisting and DbExists( mngConn, dbName ):
        DropDb( mngConn, dbName )

    if mngConn.dbms == DBMS.mySql:
        CreateMySqlDb( mngConn, dbName )
    elif mngConn.dbms == DBMS.pgSql:
        CreatePgSqlDb( mngConn, dbName )
    else:
        msg = SafeFmt( "Unsupported database management system: '%s'",
                       '/'.join( DBMS.GetNames( mngConn.dbms ) ) )
        raise DbModuleError( DbModuleError.codes.dbmsNotSupported, msg )


def CreateMySqlDb( mngConn, dbName ):
    query =  "CREATE DATABASE `{0:s}` CHARACTER SET = utf8;".format( dbName )
    mngConn.Cursor().execute( query )


def CreatePgSqlDb( mngConn, dbName ):
    import psycopg2.extensions

    # Python DB API specification expects that its implementation starts a transaction
    # automatically. On the other side, PostgreSQL's CREATE DATABASE statement mustn't be
    # executed inside of any transaction block (it causes 'InternalError: CREATE DATABASE
    # cannot run inside a transaction block' exception). To avoid it, following PostgreSQL's
    # specific extension must be used (set_isolation_level).

    prevLevel = mngConn.dbConn.isolation_level
    mngConn.dbConn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
    query = "CREATE DATABASE \"{0:s}\";".format( dbName )
    mngConn.Cursor().execute( query )
    mngConn.dbConn.set_isolation_level( prevLevel )
