#
#  FILE NAME:
#      drop.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.base import SafeFmt
from p3.db.dbms import DBMS
from p3.db.error import DbModuleError
from p3.db.manage.check import DbExists


_LOG = logging.getLogger( "p3.db" )


## \param mngConn DbConnection object which will be used for database management tasks;
#  \param dbName name of a database to be dropped;
#  \param skipNotExisting flag indicating that the function should attempt database drop only
#      when it exists;
def DropDb( mngConn, dbName, skipNotExisting=False ):
    _LOG.info( "Dropping '%s' database...", dbName )

    assert isinstance( dbName, str )

    if skipNotExisting and not DbExists( mngConn, dbName ):
        _LOG.info( "Skipping droping of not existing '%s' database.", dbName )
        return

    if mngConn.dbms == DBMS.mySql:
        DropMySqlDb( mngConn, dbName )
    elif mngConn.dbms == DBMS.pgSql:
        DropPgSqlDb( mngConn, dbName )
    else:
        msg = SafeFmt( "Unsupported database management system: '%s'",
                       '/'.join( DBMS.GetNames( mngConn.dbms ) ) )
        raise DbModuleError( DbModuleError.codes.dbmsNotSupported, msg )


def DropMySqlDb( mngConn, dbName ):
    query =  "DROP DATABASE `{0:s}`;".format( dbName )
    mngConn.Cursor().execute( query )


def DropPgSqlDb( mngConn, dbName ):
    import psycopg2.extensions

    # To understand why set_isolation_level method is invoked, see comment in the
    # 'CreatePgSqlDb' function.

    prevLevel = mngConn.dbConn.isolation_level
    mngConn.dbConn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
    query =  "DROP DATABASE \"{0:s}\";".format( dbName )
    mngConn.Cursor().execute( query )
    mngConn.dbConn.set_isolation_level( prevLevel )
