#
#  FILE NAME:
#      date_converter.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import datetime
import logging
import numbers
import re

# P3 imports:
from p3.base import Enum, SafeFmt
from p3.logic.var.converter.conversion_result import ConversionResult

_LOG = logging.getLogger( "p3.logic" )


class DateConverter( object ):
    errors = Enum( "emptyInput", "invalidFormat", "invalidDate", "maxExceeded", "minExceeded" )
    formats = Enum( "ddmmyy", "ddmmyyyy", "yymmdd", "yyyymmdd" )

    ## ===[ Initialization ]==================================================================== ##

    def __init__( self ):
        self.format = self.formats.yyyymmdd
        self.pattern = None
        self.maxVal  = None
        self.minVal  = None


    def Format( self, fmt ):
        assert fmt in self.formats

        self.format = fmt
        return self


    def Max( self, val ):
        self.maxVal = val
        return self


    def Min( self, val ):
        self.minVal = val
        return self


    ## ===[ Conversion ]======================================================================== ##

    def Deserialize( self, value ):
        if not isinstance( value, basestring ):
            raise TypeError( SafeFmt( "DateConverter.Deserialize methods expects basestring as its"
                                      " parameter (got %s instead).", type(value) ) )

        res = ConversionResult( value, self.__class__ )

        if not value:
            res.errors.append( self.errors.emptyInput )
            return res

        if self.pattern is None:
            self.pattern = self._BuildPattern( self.format )
        m = self.pattern.match( value )

        if m is not None:
            if m.group("sep1") != m.group("sep2"):
                res.errors.append( self.errors.invalidFormat )
            else:
                if self.format in (self.formats.ddmmyy, self.formats.yymmdd):
                    y = 2000+int(m.group("year"))
                else:
                    y = int(m.group("year"))
                try:
                    outVal = datetime.date( y, int(m.group("month")), int(m.group("day")) )
                except ValueError as e:
                    res.errors.append( self.errors.invalidDate )
                else:
                    if (((self.minVal is None) or (outVal >= self.minVal)) and
                        ((self.maxVal is None) or (outVal < self.maxVal))):
                        res.outVal = outVal
                    elif (self.minVal is not None) and (outVal < self.minVal):
                        res.errors.append( self.errors.minExceeded )
                    elif self.maxVal is not None and outVal >= self.maxVal:
                        res.errors.append( self.errors.maxExceeded )
        else:
            res.errors.append( self.errors.invalidFormat )

        return res

    @classmethod
    def IsDateLike( self, obj ):
        return (hasattr(obj, 'year') and hasattr(obj, 'month') and hasattr(obj, 'day') and
                isinstance(obj.year, numbers.Integral) and
                isinstance(obj.month, numbers.Integral) and
                isinstance(obj.day, numbers.Integral))

    def Serialize( self, value ):
        if not self.IsDateLike( value ):
            raise TypeError( SafeFmt( "DateConverter.Serialize methods expects date like object as"
                                      " its parameter (got %s instead).", type(value) ) )

        res = ConversionResult( value, self.__class__ )

        try:
            datetime.date( value.year, value.month, value.day )
        except ValueError as e:
            res.errors.append( self.errors.invalidDate )
        else:
            fmt = { self.formats.ddmmyy:   "{d:02d}/{m:02d}/{y:02d}",
                    self.formats.ddmmyyyy: "{d:02d}/{m:02d}/{y:04d}",
                    self.formats.yymmdd:   "{y:02d}/{m:02d}/{d:02d}",
                    self.formats.yyyymmdd: "{y:04d}/{m:02d}/{d:02d}" }.get( self.format )
            assert fmt is not None

            if self.format in (self.formats.ddmmyy, self.formats.yymmdd):
                y = value.year % 100
            else:
                y = value.year

            res.outVal = fmt.format( y=y, m=value.month, d=value.day )

        return res


    ## ===[ Internals ]========================================================================= ##

    def _BuildPattern( self, fmt ):
        dayGrp = "(?P<day>\d{2})"   # day
        mthGrp = "(?P<month>\d{2})" # month
        syrGrp = "(?P<year>\d{2})"  # short year
        lyrGrp = "(?P<year>\d{4})"  # long year
        sep    = "[-/.]{0,1}"       # separator (in future, allowed separators can depend on
                                    # converter configuration)
        sep1 = "(?P<sep1>%s)" % sep
        sep2 = "(?P<sep2>%s)" % sep

        if self.format == self.formats.ddmmyy:
            return re.compile( "^%s%s%s%s%s$" % (dayGrp,sep1,mthGrp,sep2,syrGrp) )
        elif self.format == self.formats.ddmmyyyy:
            return re.compile( "^%s%s%s%s%s$" % (dayGrp,sep1,mthGrp,sep2,lyrGrp) )
        elif self.format == self.formats.yymmdd:
            return re.compile( "^%s%s%s%s%s$" % (syrGrp,sep1,mthGrp,sep2,dayGrp) )
        elif self.format == self.formats.yyyymmdd:
            return re.compile( "^%s%s%s%s%s$" % (lyrGrp,sep1,mthGrp,sep2,dayGrp) )
