#
#  FILE NAME:
#      env.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging
import sys
import unittest

# P3 imports:
from p3.conf import ConfigureLogger, ModuleSettings
from p3.test.error import TestModuleError


_LOG = logging.getLogger( "p3.test" )


class TestEnv( object ):
    def __init__( self, settingsName, createSuiteCb ):
        assert callable(createSuiteCb)

        self.createSuiteCb = createSuiteCb
        self.options = None
        self.settings = ModuleSettings( settingsName )

        ConfigureLogger( self.settings )


    def ParseOptions( self ):
        if sys.version_info >= (2,7):
            return self._ParseOptions_ArgParse()
        elif sys.version_info >= (2,3):
            return self._ParseOptions_OptParse()
        else:
            raise TestModuleError( TestModuleError.codes.unsupportedPythonVersion,
                                   "TestEnv class requires Python 2.3 or newer." )


    def CreateSelectiveSuite( self ):
        assert self.options is not None

        return unittest.defaultTestLoader.loadTestsFromNames( self.options.tests )


    def Execute( self ):
        self.options = self.ParseOptions()
        _LOG.info( "Parsed program options: %s", self.options )

        if self.options.tests:
            s = self.CreateSelectiveSuite()
        else:
            s = self.createSuiteCb()

        unittest.TextTestRunner( verbosity=self.options.verbosity ).run( s )


    # ===[ Internal ]============================================================================ #

    def _ParseOptions_ArgParse( self ):
        import argparse

        parser = argparse.ArgumentParser( description="Database upgrade tool test runner." )

        parser.add_argument( "--verbosity",
                             action='store',
                             default=2,
                             type=int,
                             choices=(0,1,2),
                             help="Standard test runner verbosity as expected by unittest."
                             "TextTestRunner class." )

        parser.add_argument( "tests",
                             action='store',
                             type=str,
                             nargs='*',
                             help="Optional list of tests to be executed (if no test is specified"
                             " all tests will be executed)." )

        opts = parser.parse_args()
        return opts


    def _ParseOptions_OptParse( self ):
        import optparse

        def VerbosityCb( option, optStr, value, parser ):
            setattr( parser.values, option.dest, int(value) )

        parser = optparse.OptionParser()
        parser.add_option( "--verbosity",
                           action='callback',
                           dest="verbosity",
                           callback=VerbosityCb,
                           type="choice",
                           choices=("0","1","2"),
                           default=2,
                           help="Standard test runner verbosity as expected by unittest."
                             "TextTestRunner class." )

        options, args = parser.parse_args()
        options.tests = args
        return options
